package SOAP::WSDL::XSD::Typelib::Builtin::NMTOKENS;
use strict;
use warnings;
use Class::Std::Fast::Storable constructor => 'none', cache => 1;

use base qw(SOAP::WSDL::XSD::Typelib::Builtin::list
      SOAP::WSDL::XSD::Typelib::Builtin::NMTOKEN);

Class::Std::initialize();

1;
